@@ -1,12 +1,12 @@
 /************************************************************************
  *	Custom standard-io library					*
  *									*
- *	Copyright (c) 1990-1994, S.R. van den Berg, The Netherlands	*
+ *	Copyright (c) 1990-1995, S.R. van den Berg, The Netherlands	*
  *	#include "../README"						*
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: cstdio.c,v 1.28 1994/10/20 18:14:27 berg Exp $";
+ "$Id: cstdio.c,v 1.30 1995/03/20 15:30:01 berg Exp $";
 #endif
 #include "procmail.h"
 #include "robust.h"
@@ -108,7 +108,7 @@ void ungetb(x)const int x;	/* only for pushing back original characters */
      rcbufp--;							   /* backup */
 }
 
-int testb(x)const int x;	   /* fgetc that only succeeds if it matches */
+int testB(x)const int x;	   /* fgetc that only succeeds if it matches */
 { int i;
   if((i=getb())==x)
      return 1;
@@ -122,11 +122,21 @@ int sgetc P((void))				/* a fake fgetc for a string */
 
 int skipspace P((void))
 { int any=0;
-  while(testb(' ')||testb('\t'))
+  while(testB(' ')||testB('\t'))
      any=1;
   return any;
 }
 
+void skipline P((void))
+{ for(;;)					/* skip the rest of the line */
+     switch(getb())
+      { default:
+	   continue;
+	case '\n':case EOF:
+	   return;
+      }
+}
+
 void getlline(target)char*target;
 { char*chp2;
   for(;getbl(chp2=target)&&*(target=strchr(target,'\0')-1)=='\\';
