@@ -1,4 +1,4 @@
-/*$Id: includes.h,v 1.50 1994/10/20 18:14:29 berg Exp $*/
+/*$Id: includes.h,v 1.54 1995/05/17 09:23:47 berg Exp $*/
 
 #include "../autoconf.h"
 #ifdef NO_const
@@ -79,8 +79,8 @@
 #endif
 #ifndef SYSLOG_H_MISSING
 #include <syslog.h>		/* openlog() syslog() closelog() LOG_EMERG
-				/* LOG_ALERT LOG_ERR LOG_NOTICE LOG_PID
-				   LOG_MAIL */
+				/* LOG_ALERT LOG_CRIT LOG_ERR LOG_NOTICE
+				   LOG_PID LOG_MAIL */
 #endif
 #include <errno.h>		/* EINTR EEXIST ENFILE EACCES EAGAIN EXDEV */
 				/* EDQUOT ENOSPC */
@@ -308,6 +308,7 @@ extern int errno;
 #define syslog				(void)
 #define closelog()
 #define LOG_EMERG			0
+#define LOG_CRIT			0
 #define LOG_ALERT			0
 #define LOG_ERR				0
 #define LOG_NOTICE			0
@@ -397,6 +398,32 @@ extern void*memmove();
 #undef NOsetresgid
 #endif
 
+#ifdef setrgid_BRAIN_DAMAGE
+#undef setrgid_BRAIN_DAMAGE
+#ifdef setrgid
+#undef setrgid
+#endif
+#ifdef setruid
+#undef setruid
+#endif
+#define setrgid(gid)	(-1)		   /* and you think POSIX is broken? */
+#define setruid(uid)	(-1)			   /* BSD 4.4 just topped it */
+#endif
+
+#ifdef setrgid_RUNTIME_CHECK
+#undef setrgid_RUNTIME_CHECK
+#define setRgid(gid)	(setrgid(gid)||getgid()!=(gid))
+#define setRuid(uid)	(setruid(uid)||getuid()!=(uid))
+#else
+#define setRgid(gid)	setrgid(gid)
+#define setRuid(uid)	setruid(uid)
+#endif
+
+#ifdef NOinitgroups
+#undef NOinitgroups
+#define initgroups(n,g)
+#endif
+
 #ifdef NOpow
 #define tpow(x,y)	(x)
 #else
