@@ -8,14 +8,14 @@
  *									*
  *	Seems to be relatively bug free.				*
  *									*
- *	Copyright (c) 1990-1994, S.R. van den Berg, The Netherlands	*
+ *	Copyright (c) 1990-1995, S.R. van den Berg, The Netherlands	*
  *	#include "../README"						*
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: lockfile.c,v 1.30 1994/10/14 18:43:30 berg Exp $";
+ "$Id: lockfile.c,v 1.33 1995/10/30 02:09:21 srb Exp $";
 #endif
-static /*const*/char rcsdate[]="$Date: 1994/10/14 18:43:30 $";
+static /*const*/char rcsdate[]="$Date: 1995/10/30 02:09:21 $";
 #include "includes.h"
 #include "sublib.h"
 #include "exopen.h"
@@ -25,7 +25,7 @@ static /*const*/char rcsdate[]="$Date: 1994/10/14 18:43:30 $";
 #define SYSTEM_MBOX	SYSTEM_MAILBOX
 #endif
 
-static volatile exitflag;
+static volatile int exitflag;
 pid_t thepid;
 uid_t uid;
 gid_t sgid;
@@ -231,7 +231,7 @@ outofmem:	 retval=EX_OSERR,nlog("Out of memory");
 #ifdef EDQUOT
 	      case EDQUOT:
 #endif
-	      case ENOENT:case ENOTDIR:case EIO:case EACCES:
+	      case ENOENT:case ENOTDIR:case EIO:/*case EACCES:*/
 		 if(!--permanent)	 /* NFS sporadically generates these */
 		  { sleep(sleepsec);	/* unwarranted, so ignore them first */
 		    continue;
